@@ -47,45 +47,7 @@ class InvalidInterpolation(Exception):  # noqa: N818
 
 
 class Interpolator:
-    """Configuration options may contain environment variables.
-
-    For example, suppose the shell contains ``VERIFIER_NAME=testinfra`` and
-    the following molecule.yml is supplied.
-
-    ```yaml
-        verifier:
-          - name: ${VERIFIER_NAME}
-    ```
-
-    Molecule will substitute ``$VERIFIER_NAME`` with the value of the
-    ``VERIFIER_NAME`` environment variable.
-
-    !!! warning
-
-        If an environment variable is not set, Molecule substitutes with an
-        empty string.
-
-    Both ``$VARIABLE`` and ``${VARIABLE}`` syntax are supported. Extended
-    shell-style features, such as ``${VARIABLE-default}`` and
-    ``${VARIABLE:-default}`` are also supported. Even the default as another
-    environment variable is supported like ``${VARIABLE-$DEFAULT}`` or
-    ``${VARIABLE:-$DEFAULT}``. An empty string is returned when
-    both variables are undefined.
-
-    If a literal dollar sign is needed in a configuration, use a double dollar
-    sign (`$$`).
-
-    Molecule will substitute special ``MOLECULE_`` environment variables
-    defined in `molecule.yml`.
-
-    !!! note
-
-        Remember, the ``MOLECULE_`` namespace is reserved for Molecule.  Do not
-        prefix your own variables with `MOLECULE_`.
-
-    A file may be placed in the root of the project as `.env.yml`, and Molecule
-    will read variables when rendering `molecule.yml`.  See command usage.
-    """
+    """The Interpolator class."""
 
     def __init__(
         self,
