@@ -30,24 +30,29 @@ import sys
 from pathlib import Path
 from typing import TYPE_CHECKING, overload
 
+import click
 import jinja2
 import yaml
 
 from ansible_compat.ports import cache
-from rich.syntax import Syntax
 
-from molecule.console import console
-from molecule.constants import MOLECULE_HEADER
+from molecule.constants import (
+    MOLECULE_COLLECTION_GLOB,
+    MOLECULE_COLLECTION_ROOT,
+    MOLECULE_GLOB,
+    MOLECULE_HEADER,
+    MOLECULE_ROOT,
+)
 from molecule.exceptions import MoleculeError
 
 
 if TYPE_CHECKING:
     from collections.abc import Generator, Iterable, MutableMapping, Sequence
     from io import TextIOWrapper
-    from typing import Any, AnyStr, NoReturn, TypeVar
+    from typing import Any, NoReturn, TypeVar
     from warnings import WarningMessage
 
-    from molecule.types import CommandArgs, ConfigData, Options, PlatformData
+    from molecule.types import CollectionData, CommandArgs, ConfigData, Options, PlatformData
 
     NestedDict = MutableMapping[str, Any]
     _T = TypeVar("_T", bound=NestedDict)
@@ -73,47 +78,27 @@ class SafeDumper(yaml.SafeDumper):
         return super().increase_indent(flow, indentless=False)
 
 
-def print_debug(title: str, data: str) -> None:
-    """Print debug information.
-
-    Args:
-        title: A title to describe the data.
-        data: The data to print.
-    """
-    console.print(f"DEBUG: {title}:\n{data}")
-
-
 def print_environment_vars(env: dict[str, str] | None) -> None:
-    """Print ``Ansible`` and ``Molecule`` environment variables and returns None.
+    """Log ``Ansible`` and ``Molecule`` environment variables and returns None.
 
     Args:
         env: A dict containing the shell's environment as collected by ``os.environ``.
     """
-    if env:
-        ansible_env = {k: v for (k, v) in env.items() if "ANSIBLE_" in k}
-        print_debug("ANSIBLE ENVIRONMENT", safe_dump(ansible_env, explicit_start=False))
+    if not env:
+        return
 
-        molecule_env = {k: v for (k, v) in env.items() if "MOLECULE_" in k}
-        print_debug(
-            "MOLECULE ENVIRONMENT",
-            safe_dump(molecule_env, explicit_start=False),
-        )
+    logger = logging.getLogger(__name__)
 
-        combined_env = ansible_env.copy()
-        combined_env.update(molecule_env)
-        print_debug(
-            "SHELL REPLAY",
-            " ".join([f"{k}={v}" for (k, v) in sorted(combined_env.items())]),
-        )
+    sections: dict[str, list[tuple[str, str]]] = {}
 
+    for n in ["ANSIBLE", "MOLECULE"]:
+        sections[n] = [(k, v) for (k, v) in sorted(env.items()) if k.startswith(f"{n}_")]
+        logger.debug("%s ENVIRONMENT:\n%s\n", n, "\n".join(f"{k}: {v}" for (k, v) in sections[n]))
 
-def do_report() -> None:
-    """Dump html report atexit."""
-    report_file = Path(os.environ["MOLECULE_REPORT"])
-    LOG.info("Writing %s report.", report_file)
-    with report_file.open("w") as f:
-        f.write(console.export_html())
-        f.close()
+    logger.debug(
+        "SHELL REPLAY: %s",
+        " ".join(f"{k}={v}" for _, es in sections.items() for (k, v) in es),
+    )
 
 
 def sysexit(code: int = 1) -> NoReturn:
@@ -130,17 +115,45 @@ def sysexit_with_message(
     code: int = 1,
     warns: Sequence[WarningMessage] = (),
 ) -> NoReturn:
-    """This method is a lie for compatibility purposes.
+    """Wrapper around sysexit to also display a message.
 
     Args:
         msg: The message to display.
         code: The return code to exit with.
         warns: A series of warnings to send alongside the message.
-
-    Raises:
-        MoleculeError: always.
     """
-    raise MoleculeError(message=msg, code=code, warns=warns)
+    for warning in warns:
+        LOG.warning(warning.message)
+
+    # For success (code 0), always use info logging
+    # For failures (code != 0), use debug-aware logging
+    if code == 0:
+        LOG.info(msg)
+    else:
+        # Show only the error message in normal mode for failures
+        LOG.error(msg)
+
+    sysexit(code)
+
+
+def sysexit_from_exception(exc: MoleculeError) -> NoReturn:
+    """Wrapper for sysexit to display messages and use return code from an exception.
+
+    Args:
+        exc: The exception to determine exit values from.
+    """
+    # Check if debug mode is enabled
+    ctx = click.get_current_context(silent=True)
+    debug_mode = False
+    if ctx and ctx.obj and isinstance(ctx.obj, dict):
+        debug_mode = ctx.obj.get("args", {}).get("debug", False)
+
+    if debug_mode:
+        # Show full traceback in debug mode for failures
+        LOG.exception(exc.message)
+        sysexit(exc.code)
+    else:
+        sysexit_with_message(exc.message, exc.code)
 
 
 def os_walk(
@@ -205,6 +218,7 @@ def write_file(filename: str | Path, content: str, header: str | None = None) ->
 
     if isinstance(filename, str):
         filename = Path(filename)
+    filename.parent.mkdir(exist_ok=True)
     filename.write_text(content)
 
 
@@ -458,31 +472,41 @@ def lookup_config_file(filename: str) -> str | None:
     """Return config file PATH.
 
     Args:
-        filename: Config file name to find.and
+        filename: Config file name to find.
 
     Returns:
         Path to config file or None if not found.
     """
-    for path in [find_vcs_root(default="~"), "~"]:
-        f = (Path(path) / filename).expanduser()
-        if f.is_file():
-            LOG.info("Found config file %s", f)
-            return str(f)
+    # project root
+    search_paths = [Path(find_vcs_root(default="~")) / filename]
+
+    # collection path
+    collection_dir, _ = get_collection_metadata()
+    if collection_dir:
+        search_paths.append(Path(collection_dir / MOLECULE_COLLECTION_ROOT / Path(filename).name))
+
+    # home directory
+    search_paths.append(Path.home() / filename)
+
+    for path in search_paths:
+        if path.is_file():
+            LOG.info("Found config file %s", path)
+            return str(path)
     return None
 
 
-def boolean(value: bool | AnyStr, *, strict: bool = True) -> bool:
+def boolean(value: object, *, default: bool | None = None) -> bool:
     """Evaluate any object as boolean matching ansible behavior.
 
     Args:
         value: The value to evaluate as a boolean.
-        strict: If True, invalid booleans will raises TypeError instead of returning False.
+        default: If provided, return this value for invalid inputs instead of raising TypeError.
 
     Returns:
-        The boolean value of value.
+        The boolean value of value, or default if value is invalid and default is provided.
 
     Raises:
-        TypeError: If value does not resolve to a valid boolean and strict is True.
+        TypeError: If value does not resolve to a valid boolean and no default is provided.
     """
     # Based on https://github.com/ansible/ansible/blob/devel/lib/ansible/module_utils/parsing/convert_bool.py
 
@@ -501,9 +525,13 @@ def boolean(value: bool | AnyStr, *, strict: bool = True) -> bool:
 
     if normalized_value in BOOLEANS_TRUE:
         return True
-    if normalized_value in BOOLEANS_FALSE or not strict:
+    if normalized_value in BOOLEANS_FALSE:
         return False
 
+    # If we have a default, return it for invalid values
+    if default is not None:
+        return default
+
     raise TypeError(  # noqa: TRY003
         f"The value '{value!s}' is not a valid boolean.  Valid booleans include: {', '.join(repr(i) for i in BOOLEANS)!s}",  # noqa: EM102
     )
@@ -530,7 +558,7 @@ def dict2args(data: MutableMapping[str, str | bool]) -> list[str]:
     return result
 
 
-def bool2args(data: bool | list[str]) -> list[str]:  # noqa: ARG001
+def bool2args(data: bool | list[str]) -> list[str]:  # noqa: ARG001, FBT001
     """Convert a boolean value to command line argument (flag).
 
     Args:
@@ -542,17 +570,6 @@ def bool2args(data: bool | list[str]) -> list[str]:  # noqa: ARG001
     return []
 
 
-def print_as_yaml(data: object) -> None:
-    """Render python object as yaml on console.
-
-    Args:
-        data: A YAML object.
-    """
-    # https://github.com/Textualize/rich/discussions/990#discussioncomment-342217
-    result = Syntax(code=safe_dump(data), lexer="yaml", background_color="default")
-    console.print(result)
-
-
 def oxford_comma(listed: Iterable[bool | str | Path], condition: str = "and") -> str:
     """Format a list into a sentence.
 
@@ -572,3 +589,90 @@ def oxford_comma(listed: Iterable[bool | str | Path], condition: str = "and") ->
             return f"{', '.join(s for s in front)}, {condition} {back}"
         case _:
             return ""
+
+
+@cache
+def get_collection_metadata() -> tuple[Path, CollectionData] | tuple[None, None]:
+    """Get collection directory and metadata.
+
+    Returns:
+        Tuple of (collection_directory, collection_data) if in a valid collection,
+        or (None, None) if not in collection or galaxy.yml is invalid.
+        Collection data dict contains at least 'name' and 'namespace' keys.
+        Only returns collection info when galaxy.yml is valid.
+    """
+    cwd = Path.cwd()
+    galaxy_file = cwd / "galaxy.yml"
+
+    try:
+        galaxy_data: CollectionData = safe_load_file(galaxy_file)
+        important_keys = {"name", "namespace"}
+
+        if not isinstance(galaxy_data, dict):
+            LOG.warning("Invalid galaxy.yml format at %s", galaxy_file)
+            return None, None
+
+        if missing_keys := important_keys.difference(galaxy_data.keys()):
+            LOG.warning(
+                "galaxy.yml at %s is missing required fields: %s",
+                galaxy_file,
+                oxford_comma(missing_keys),
+            )
+            return None, None
+    except FileNotFoundError:
+        LOG.debug("No galaxy.yml found at %s", galaxy_file)
+        return None, None
+    except (OSError, yaml.YAMLError, MoleculeError) as exc:
+        LOG.warning("Failed to load galaxy.yml at %s: %s", galaxy_file, exc)
+        return None, None
+    else:
+        LOG.debug("Found galaxy.yml at %s", galaxy_file)
+        return cwd, galaxy_data
+
+
+@cache
+def get_effective_molecule_glob() -> str:
+    """Get the appropriate glob pattern.
+
+    Returns:
+        Glob pattern string for finding molecule.yml files.
+        Returns MOLECULE_COLLECTION_GLOB if in collection,
+        otherwise returns MOLECULE_GLOB.
+    """
+    # User provided
+    if "MOLECULE_GLOB" in os.environ:
+        return os.environ["MOLECULE_GLOB"]
+
+    # No collection detected
+    collection_dir, collection_data = get_collection_metadata()
+    if not collection_dir or not collection_data:
+        return MOLECULE_GLOB
+
+    # Molecule found in root of collection
+    if (Path.cwd() / MOLECULE_ROOT).exists():
+        msg = f"Molecule scenarios should migrate to '{MOLECULE_COLLECTION_ROOT}'"
+        LOG.warning(msg)
+        return MOLECULE_GLOB
+
+    # Molecule not found in collection use extensions
+    msg = f"Collection '{collection_data['namespace']}.{collection_data['name']}' detected."
+    LOG.info(msg)
+    msg = f"Scenarios will be used from '{MOLECULE_COLLECTION_ROOT}'"
+    LOG.info(msg)
+    return MOLECULE_COLLECTION_GLOB
+
+
+def to_bool(a: object) -> bool:
+    """Return a bool for the arg.
+
+    Args:
+        a: A value to coerce to bool.
+
+    Returns:
+        A bool representation of a.
+    """
+    if a is None or isinstance(a, bool):
+        return bool(a)
+    if isinstance(a, str):
+        a = a.lower()
+    return a in ("yes", "on", "1", "true", 1)
